# -*- coding: utf-8 -*-

from odoo import http
from odoo import SUPERUSER_ID
from odoo.http import request, Response
from odoo.addons.mail.controllers.bus import MailChatController
import logging

_logger = logging.getLogger(__name__)


class ControllerREST(MailChatController):

    # http://localhost:8069/api/kazoo?kazoo_id=string&user_id=string&callerid=string

    @http.route('/api/kazoo', methods=['POST'], type='http', auth='public', csrf=False)
    def api_kazoo_call(self, **post):
        kazoo_id = post.get('kazoo_id')
        kazoo_user_id = post.get('user_id')
        callerid = post.get('callerid')
        if not kazoo_id or not kazoo_user_id or not callerid:
            return Response('Not Enough Arguments Provided.', status=400)
        base_url = request.env['ir.config_parameter'].sudo().get_param('web.base.url')
        _logger.info('Kazoo API Call: kazoo_id: %s, user_id: %s, callerid: %s', kazoo_id, kazoo_user_id, callerid)
        phone_domain = []
        mobile_domain = []
        caller = callerid.replace(' ', '')
        for i in caller:
            phone_domain.append(('phone', 'like', '%'+i+'%'))
            mobile_domain.append(('mobile', 'like', '%'+i+'%'))
        caller_partner_phone = request.env['res.partner'].sudo().search(phone_domain)
        caller_partner_mobile = request.env['res.partner'].sudo().search(mobile_domain)
        caller_partner_id = False
        if not caller_partner_id and caller_partner_phone:
            caller_partner_id = caller_partner_phone.filtered(lambda partner: ''.join([p for p in partner.phone if p.isdigit()]) == caller)
        if not caller_partner_id and caller_partner_mobile:
            caller_partner_id = caller_partner_mobile.filtered(lambda partner: ''.join([m for m in partner.mobile if m.isdigit()]) == caller)
        if not caller_partner_id:
            return Response('Partner Not Found.', status=400)
        _logger.info('Kazoo API Call: Calling Partner ID: %s, Calling Partner Name: %s', caller_partner_id.id, caller_partner_id.name)
        partner_id_call = caller_partner_id.id
        partner_name_call = caller_partner_id.name
        res_user_id = request.env['res.users'].sudo().search([('kazoo_id', '=', kazoo_id), ('kazoo_user_id', '=', kazoo_user_id)], limit=1)
        if not res_user_id:
            return Response('User Not Found.', status=404)
        _logger.info('Kazoo API Call: User ID: %s (%s), User-Partner ID: %s (%s)', res_user_id.id, res_user_id.name, res_user_id.partner_id.id, res_user_id.partner_id.name)
        user_id = res_user_id.id
        partner_id = res_user_id.partner_id.id
        user = request.env['res.users'].sudo().browse([user_id])
        pinned_channels = request.env['mail.channel.partner'].sudo().search([('partner_id', '=', partner_id)])
        super_user = request.env['res.users'].sudo().browse([SUPERUSER_ID])
        if not pinned_channels:
            direct_message_channels = request.env["mail.channel"].with_context(mail_create_nosubscribe=False).sudo().create({
                    'channel_partner_ids': [(4, super_user.partner_id.id), (4, user.partner_id.id)],
                    'channel_type': 'chat',
                    'name': ', '.join([super_user.name, user.name]),
                    'public': 'private',
                    'email_send': False,
                })
            pinned_channels = request.env['mail.channel.partner'].sudo().create({
                    'partner_id': partner_id,
                    'channel_id': direct_message_channels.id,
                })
        pinned_channels = pinned_channels.mapped('channel_id')

        direct_message_channels = pinned_channels.sudo().filtered(lambda channel: channel.name == ', '.join([super_user.name, user.name]))

        if not direct_message_channels:
            direct_message_channels = request.env["mail.channel"].with_context(mail_create_nosubscribe=False).sudo().create({
                'channel_partner_ids': [(4, super_user.partner_id.id), (4, user.partner_id.id)],
                'channel_type': 'chat',
                'name': ', '.join([super_user.name, user.name]),
                'public': 'private',
                'email_send': False,
            })
        for pinned in direct_message_channels:
            request.session.uid = SUPERUSER_ID
            msg = """
            %s/web#id=%s&view_type=form&model=res.partner

            %s is Calling""" % (base_url, partner_id_call, partner_name_call)
            self.mail_chat_post(pinned.uuid, msg)
        return Response('Success.', status=200)
